﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Security.Cryptography.X509Certificates;
using System.Net;
using System.Xml.Linq;
using System.IO;

namespace WebRole1
{
    public static class ServiceManagementRestHelper
    {
        private static string versionNumber = "2011-10-01";
        private static XNamespace wa = "http://schemas.microsoft.com/windowsazure";
        private static string getHostedServicesOperationUrlTemplate = "https://management.core.windows.net/{0}/services/hostedservices";
        private static string getHostedServicePropertyOperationUrlTemplate = "https://management.core.windows.net/{0}/services/hostedservices/{1}?embed-detail=true";

        public static IList<string> GetHostedServices(string subscriptionId, X509Certificate2 certificate)
        {
            List<string> hostedServiceNames = new List<string>();
            string uri = string.Format(getHostedServicesOperationUrlTemplate, subscriptionId);
            XElement xe = PerformGetOperation(uri, certificate);
            if (xe != null)
            {
                var serviceNameElements = xe.Elements().Elements(XName.Get("ServiceName", wa.ToString()));
                foreach (var serviceElement in serviceNameElements)
                {
                    hostedServiceNames.Add(serviceElement.Value);
                }
            }
            return hostedServiceNames;
        }

        public static XElement GetHostedServiceProperties(string subscriptionId, X509Certificate2 certificate, string hostedServiceName)
        {
            string uri = string.Format(getHostedServicePropertyOperationUrlTemplate, subscriptionId, hostedServiceName);
            return PerformGetOperation(uri, certificate);
        }

        private static HttpWebRequest CreateHttpWebRequest(Uri uri, X509Certificate2 certificate, String httpWebRequestMethod)
        {
            HttpWebRequest httpWebRequest = (HttpWebRequest)HttpWebRequest.Create(uri);
            httpWebRequest.Method = httpWebRequestMethod;
            httpWebRequest.Headers.Add("x-ms-version", versionNumber);
            httpWebRequest.ClientCertificates.Add(certificate);
            httpWebRequest.ContentType = "application/xml";
            return httpWebRequest;
        }

        private static XElement PerformGetOperation(string uri, X509Certificate2 certificate)
        {
            XElement responseBody = null;
            Uri requestUri = new Uri(uri);
            HttpWebRequest httpWebRequest = CreateHttpWebRequest(requestUri, certificate, "GET");
            using (HttpWebResponse response = (HttpWebResponse)httpWebRequest.GetResponse())
            {
                Stream responseStream = response.GetResponseStream();
                responseBody = XElement.Load(responseStream);
            }
            return responseBody;
        }
    }
}