using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.Security.Cryptography.X509Certificates;
using System.Net;
using System.Xml.Linq;
using System.IO;
using System.Text;

namespace WebRole1
{
    public class WebRole : RoleEntryPoint
    {
        public override bool OnStart()
        {
            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.
            return base.OnStart();
        }

        public static RoleDetails GetRoleDetails()
        {
            RoleDetails roleDetails = null;
            string subscriptionId = RoleEnvironment.GetConfigurationSettingValue("SubscriptionId");
            string certificateData = RoleEnvironment.GetConfigurationSettingValue("ManagementCertificate");
            X509Certificate2 certificate = new X509Certificate2(Convert.FromBase64String(certificateData));
            IList<string> hostedServices = ServiceManagementRestHelper.GetHostedServices(subscriptionId, certificate);
            string deploymentId = RoleEnvironment.DeploymentId;
            if (hostedServices != null && hostedServices.Count > 0)
            {
                foreach (var hostedService in hostedServices)
                {
                    XElement xe = ServiceManagementRestHelper.GetHostedServiceProperties(subscriptionId, certificate, hostedService);
                    if (xe != null)
                    {
                        var deploymentXElements = xe.Elements(XName.Get("Deployments", "http://schemas.microsoft.com/windowsazure")).Elements(XName.Get("Deployment", "http://schemas.microsoft.com/windowsazure")).ToList();
                        if (deploymentXElements != null && deploymentXElements.Count > 0)
                        {
                            foreach (var deployment in deploymentXElements)
                            {
                                string currentDeploymentId = deployment.Element(XName.Get("PrivateID", "http://schemas.microsoft.com/windowsazure")).Value;
                                if (currentDeploymentId == deploymentId)
                                {
                                    string deploymentSlotName = deployment.Element(XName.Get("DeploymentSlot", "http://schemas.microsoft.com/windowsazure")).Value;
                                    roleDetails = new RoleDetails()
                                    {
                                        HostedServiceName = hostedService,
                                        DeploymentSlot = deploymentSlotName,
                                        DeploymentId = RoleEnvironment.DeploymentId,
                                    };
                                    break;
                                }
                            }
                        }
                    }
                    if (roleDetails != null)
                    {
                        break;
                    }
                }
            }
            return roleDetails;

        }
    }

    public class RoleDetails
    {
        public string HostedServiceName
        {
            get;
            set;
        }

        public string DeploymentSlot
        {
            get;
            set;
        }

        public string DeploymentId
        {
            get;
            set;
        }
    }
}
